#ifndef LIVEIR_SDK_H
#define LIVEIR_SDK_H
/***************************************************************************
                          liveir.h  -  Live!Drive IR SDK Header file
                             -------------------
    begin                : Mon Dec 24 2001
    copyright            : (C) 2001 by Stephen Evanchik
    email                : evanchsa@clarkson.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>

/* Global data structures
 * LiveDrive IR device structure
 */
struct liveir_dev
{
	/* Configuration varibles */
	int remote_type;

	/* IR port access varibles */
	FILE *fstream;

	char *device_file;
};

struct liveir_remote
{
	int remote_type;
	char *remote_name;

	int num_keys;
	struct remote_control_key *keys;
};

/* Key codes with descriptions */
struct remote_control_key
{	
	long key;
	char desc[16];
	int remote_type;
};

/* Creative MIDI Constants */
#define CL_MIDI_MFG_ID          0x05
#define CL_MIDI_DEV_ID_HIGH     0x21
#define CL_MIDI_DEV_ID_LOW      0x20

/* Remote Type Constants.
 * There are many remote controls that will work with the Live!Drive IR port.
 * I've used the Creative Infra Drive remote successfully so let's be reasonable and
 * assume that someone will want to use that remote one day.
 */
#define CL_REMOTEID_RM900           1
#define CL_REMOTEID_iR2000          2

/* Creative RM-900 Constants
 *
 *
 * The length of the command sent by the RM-900
 */
#define CL_RM900_CMD_LEN	49

/* Remote Control Key constants
 * Observations:
 * 1. The Live!Drive IR seems to output commands in 16bit words
 * 2. A command is delimited by 0xF0 and 0xF7
 * 3. The command itself is in the form 0x0005 (vendor ID) followed by <0x****, 0x005>*
 * 4. After removing the 0x0005's, all but three bytes change for each button
 * One byte is a flag of some sort (probably parity) and the other two are the actual key pressed.
 * There are two other bytes, 0x44 0x41 (0xC1 after the transformation is applied) that  form the low
 * word of the command with the upper word being the key. I think the low word is the remote type.
 * See liveird to understand how to convert the raw keycodes to these.
 */
#define CL_KEY_RM900_ONE						0x2ED144C1
#define CL_KEY_RM900_TWO						0x0EF144C1
#define CL_KEY_RM900_THREE					0xF60944C1
#define CL_KEY_RM900_FOUR						0xAE5144C1
#define CL_KEY_RM900_FIVE						0xDE2144C1
#define CL_KEY_RM900_SIX						0xE11E44C1
#define CL_KEY_RM900_SEVEN					0x6E9144C1
#define CL_KEY_RM900_EIGHT					0x3EC144C1
#define CL_KEY_RM900_NINE						0x11EE44C1
#define CL_KEY_RM900_ZERO						0xFE0144C1

#define CL_KEY_RM900_STOPEJECT			0x5EA144C1
#define CL_KEY_RM900_PLAYPAUSE			0x619E44C1
#define CL_KEY_RM900_SLOW						0x41BE44C1
#define CL_KEY_RM900_PREVIOUS				0x01FE44C1
#define CL_KEY_RM900_NEXT						0xA15E44C1
#define CL_KEY_RM900_STEP						0x817E44C1

#define CL_KEY_RM900_MUTE						0x897644C1
#define CL_KEY_RM900_VOLDOWN				0x39C644C1
#define CL_KEY_RM900_VOLUP					0xB94644C1

#define CL_KEY_RM900_EAX						0xCE3144C1
#define CL_KEY_RM900_OPTIONS				0xBE4144C1
#define CL_KEY_RM900_DISPLAY				0x916E44C1
#define CL_KEY_RM900_RETURN					0x8E7144C1
#define CL_KEY_RM900_STARTMENU			0xEE1144C1
#define CL_KEY_RM900_CLOSE					0xC13E44C1

#define CL_KEY_RM900_SELECT					0x7E8144C1
#define CL_KEY_RM900_UP							0x21DE44C1
#define CL_KEY_RM900_DOWN						0x4EB144C1
#define CL_KEY_RM900_REVERSE				0x1EE144C1
#define CL_KEY_RM900_FORWARD				0x51AE44C1


/* Creative iR2000 Constants
 *
 *
 * The length of the command sent by the iR2000
 */
#define CL_iR2000_CMD_LEN 49

#define CL_KEY_iR2000_PAUSE					0x7F80AC21
#define CL_KEY_iR2000_PLAYNEXT			0xBF40AC21
#define CL_KEY_iR2000_STOPEJECT			0x3FC0AC21
#define CL_KEY_iR2000_REVERSE				0x5FA0AC21
#define CL_KEY_iR2000_PREVIOUS			0x9F60AC21
#define CL_KEY_iR2000_FORWARD				0x1FE0AC21

#define CL_KEY_iR2000_MENU					0xDF20AC21
#define CL_KEY_iR2000_MOUSE					0xEF10AC21
#define CL_KEY_iR2000_MUTE					0xCF30AC21
#define CL_KEY_iR2000_VOLUP					0xF708AC21
#define CL_KEY_iR2000_VOLDOWN				0xD728AC21
#define CL_KEY_iR2000_MAGNIFIY			0xE718AC21

#define CL_KEY_iR2000_ONE						0x6F90AC21
#define CL_KEY_iR2000_TWO						0xAF50AC21
#define CL_KEY_iR2000_THREE					0x2FD0AC21
#define CL_KEY_iR2000_FOUR					0x4FB0AC21
#define CL_KEY_iR2000_FIVE					0x8F70AC21
#define CL_KEY_iR2000_SIX						0x0FF0AC21
#define CL_KEY_iR2000_SEVEN					0x7788AC21
#define CL_KEY_iR2000_EIGHT					0xB748AC21
#define CL_KEY_iR2000_NINE					0x37C8AC21
#define CL_KEY_iR2000_ZERO					0x9768AC21
#define CL_KEY_iR2000_SHIFT					0x57A8AC21
#define CL_KEY_iR2000_ENTER					0x17E8AC21


/* This is a list of all keys for all remotes,
 * hopefully I'll be able to add things without too much trouble
 */
static struct remote_control_key key_list[] =
{
	/* Begin RM-900 Key list */
	{ CL_KEY_RM900_ONE, 				"1",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_TWO, 				"2",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_THREE,				"3",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_FOUR, 				"4",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_FIVE,				"5",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_SIX,					"6",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_SEVEN,				"7",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_EIGHT,				"8",						CL_REMOTEID_RM900		},
	{  CL_KEY_RM900_NINE,				"9",						CL_REMOTEID_RM900		},
	{  CL_KEY_RM900_ZERO,				"0",						CL_REMOTEID_RM900		},

	{ CL_KEY_RM900_STOPEJECT,		"Stop/Eject",		CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_PLAYPAUSE,		"Play/Pause",		CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_SLOW,				"Slow",					CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_PREVIOUS,		"Previous",			CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_NEXT,				"Next",					CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_STEP,				"Step",					CL_REMOTEID_RM900		},

	{ CL_KEY_RM900_MUTE,				"Mute",					CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_VOLDOWN,			"Volume Down",	CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_VOLUP,				"Volume Up",		CL_REMOTEID_RM900		},

	{ CL_KEY_RM900_EAX,					"EAX",					CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_OPTIONS,			"Options",			CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_DISPLAY,			"Display",			CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_RETURN,			"Return",				CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_STARTMENU,		"Start Menu",		CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_CLOSE,				"Close",				CL_REMOTEID_RM900		},

	{ CL_KEY_RM900_SELECT,			"Select",    		CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_UP,					"Up",						CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_DOWN,				"Down",					CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_REVERSE,			"Reverse",			CL_REMOTEID_RM900		},
	{ CL_KEY_RM900_FORWARD,			"Forward",			CL_REMOTEID_RM900		},

	/* iR2000 Key list */
	{ CL_KEY_iR2000_PAUSE,			"Pause",				CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_PLAYNEXT,		"Play/Next",		CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_STOPEJECT,	"Stop/Eject",		CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_REVERSE,		"Reverse",			CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_PREVIOUS,		"Previous",			CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_FORWARD,		"Forward",			CL_REMOTEID_iR2000	},

	{ CL_KEY_iR2000_MENU,				"Menu",					CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_MOUSE,			"Mouse",				CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_MUTE,				"Mute",					CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_VOLUP,			"Volume Up",		CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_VOLDOWN,		"Volume Down",	CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_MAGNIFIY,		"Magnify",			CL_REMOTEID_iR2000	},

	{ CL_KEY_iR2000_ONE,				"1",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_TWO,				"2",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_THREE,			"3",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_FOUR,				"4",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_FIVE,				"5",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_SIX,				"6",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_SEVEN,			"7",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_EIGHT,			"8",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_NINE,				"9",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_ZERO,				"0",						CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_SHIFT,			"Shift",				CL_REMOTEID_iR2000	},
	{ CL_KEY_iR2000_ENTER,			"Enter",				CL_REMOTEID_iR2000	},

	/* The end of the list */
	{ 0,												"\0",						0										}

};

/* The maximum size of a message */
#define MSG_SIZE 32

/* The internal message buffer */
struct liveir_msgbuf
{
	long mtype;
	char mtext[MSG_SIZE];
};

/* A global constant used to define the message queue */
static const char MSG_PROJECT = 'L';

#endif
